//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#include "ReadAscii.h"
#include "MakeTriggers.h"
#include "Segments.h"

using namespace std;

int usage(){
    cerr<<"This script converts txt files with columns into a standard ROOT trigger file"<<endl;
    cerr<<"The txt files should provide _at least_ the trigger time and SNR"<<endl;
    cerr<<"The output root file is dumped in the current directory"<<endl;
    cerr<<endl;
    cerr<<"usage:"<<endl;
    cerr<<"TxtToTriggers.exe [input text file pattern] [output file name] [column headers] [segment file]"<<endl; 
    cerr<<endl;
    cerr<<"  [input text file] : txt file with columns of data"<<endl; 
    cerr<<"  [output file name]: base name for output file (no root extension)"<<endl; 
    cerr<<"  [column headers]  : assign column numbers"<<endl; 
    cerr<<"                      10 columns should be given in the following order:"<<endl;
    cerr<<"                      time - SNR - frequency - time start - time end - frequency start - frequency end - q - amplitude - phase"<<endl; 
    cerr<<"                      a '-1' value can be used if the paramater does not exist"<<endl; 
    cerr<<"  [segment file]    : segment file associated to triggers (optional argument)"<<endl; 
    cerr<<endl; 
    cerr<<"Example:"<<endl; 
    cerr<<"TxtToTriggers.exe \"/path/to/my/files/*.txt\" \"triggers\" \"1;5;2;-1;-1;-1;-1;9;-1;-1\" /path/to/my/segments.txt"<<endl; 
    cerr<<" - all the txt files in /path/to/my/files/ will be considered"<<endl; 
    cerr<<" - the output file will be named \"triggers\""<<endl; 
    cerr<<" - time      = column 1"<<endl; 
    cerr<<" - SNR       = column 5"<<endl; 
    cerr<<" - frequency = column 2"<<endl; 
    cerr<<" - time start= does not exist"<<endl; 
    cerr<<" - time end  = does not exist"<<endl; 
    cerr<<" - freq start= does not exist"<<endl; 
    cerr<<" - freq end  = does not exist"<<endl; 
    cerr<<" - q         = column 9"<<endl; 
    cerr<<" - amplitude = does not exist"<<endl; 
    cerr<<" - phase = does not exist"<<endl; 
    return 1;
}

int main (int argc, char* argv[]){

  // check the output directory
  if(argc<=3) return usage();

  // get txt file pattern
  string intxtpattern=(string)argv[1];
  
  //  output file name
  string outfilename=(string)argv[2];

  //  segment file name
  string segfilename="";
  if(argc>3) segfilename=(string)argv[4];

  //  column headers
  vector <string> sformat=SplitString ((string)argv[3], ';');
  if((int)sformat.size()!=10){
    cerr<<argv[0]<<": cannot read column headers"<<endl<<endl; 
    return usage();
  }

  // convert column numbers into integers
  int colnumber[10];
  for(int f=0; f<10; f++){
    istringstream iss(sformat[f]); iss>>colnumber[f];
    colnumber[f]--;
  }

  // check time and SNR
  if(colnumber[0]<0||colnumber[1]<0){
    cerr<<argv[0]<<": time and SNR columns are not given"<<endl<<endl; 
    return usage();
  }
 
  // Trigger object
  MakeTriggers *triggers = new MakeTriggers(outfilename,1);
 
  string ls;
  ls = "ls "+(string)intxtpattern;
  
  cout<<"Opening text files in "<<intxtpattern<<"..."<<endl;
    
  FILE *in;
  in = popen(ls.c_str(), "r");

  ReadAscii *ra;
  double var[10];
  string svar[10];

  // loop over txt files  
  char filename[1000];
  while(fscanf(in,"%s",filename) != EOF){
    cout<<"converting "<<filename<<" ... "<<flush;
    ra = new ReadAscii((string)filename);
   
    // check file
    if(!ra->GetNRow()){
      cerr<<"no triggers -> skip file"<<endl;
      delete ra;
      continue;
    }
    
    // loop over input files
    for(int i=0; i<ra->GetNRow(); i++){

      // read elements
      for(int f=0; f<10; f++){
	if(colnumber[f]>=0&&ra->GetElement(svar[f],i,colnumber[f])){
	  istringstream iss(svar[f]); iss>>var[f];
	}
	else
	  var[f]=0.0;// default value
      }

      // add trigger
      if(!triggers->AddTrigger(var[0],var[2],var[1],var[7],var[3],var[4],var[5],var[6],var[8],var[9])) return usage();
    }

    cout<<"OK"<<endl;
    delete ra;
  }
  
  pclose(in);

  // set segments if any
  Segments *S;
  if(segfilename.compare("")){
    S = new Segments(segfilename);
    if(S->GetNsegments()>0) triggers->Append(S);
  }

  // sort triggers
  if(!triggers->SortTriggers()) return usage();

  // save trigger file
  if(!triggers->Write(".", "root", outfilename).compare("none")) return usage();
  
  if(segfilename.compare("")) delete S;
  delete triggers;

  return 0;
}
